﻿using System;
using System.Collections.Generic;
using System.IO;
using SourceStat.Utils;

namespace SourceStat.Analysis
{
	// *************************************************************************
	/*
	                                     ProjectFile
  	                                          |
	       -----------------------------------------------------------------------
	       |                 |                |               |                  |
	  HtmlFileType     VSCSharpFileType   CSSFileTyp   JavaScriptFileType   XmlFileType
	*/

	// *************************************************************************
	// *** ProjectType *********************************************************
	// *************************************************************************
	public abstract class ProjectFile
	{
		// ---------------------------------------------------------------------
		private string fileName = null;
		private long? commentsCount = null;
		// --- File Contents ---------------------------------------------------
		private string[] contentInLines = null;
		private string content = null;
		private string contentWithoutComments = null;

		// ---------------------------------------------------------------------
		// --- VSCSharpFileType Event Handlers ---------------------------------
		// ---------------------------------------------------------------------
		public event EventHandler<LogEventArgs> LogMessage;

		// ---------------------------------------------------------------------
		// --- VSCSharpFileType Proeprties -------------------------------------
		// ---------------------------------------------------------------------
		public string FileName
		{
			get
			{
				if (fileName == null)
					throw new Exception("File was not set!");

				return fileName;
			} // get

			set
			{
				if (!File.Exists(value))
					throw new Exception("File '" + value + "' not exist!");

				if (!string.IsNullOrEmpty(fileName) && !fileName.Equals(value))
					InvalidateFileContents();

				fileName = value;
			} // set
		}

		// ---------------------------------------------------------------------
		protected string[] ContentInLines
		{
			get
			{
				if (contentInLines == null)
					contentInLines = File.ReadAllLines(FileName);

				return contentInLines;
			} // get
		}

		// ---------------------------------------------------------------------
		protected string Content
		{
			get
			{
				if (content == null)
					content = File.ReadAllText(FileName);

				return content;
			} // get
		}

		// ---------------------------------------------------------------------
		protected string ContentWithoutComments
		{
			get
			{
				if (contentWithoutComments == null)
					contentWithoutComments = CommentarsBlockCount(ref commentsCount);

				return contentWithoutComments;
			} // get
		}

		// ---------------------------------------------------------------------
		// --- ProjectFile Methods ---------------------------------------------
		// ---------------------------------------------------------------------
		public virtual void InvalidateFileContents()
		{
			contentInLines = null;
			content = null;
			contentWithoutComments = null;
		}

		// ---------------------------------------------------------------------
		protected virtual bool IsEmptyLine(string line)
		{
			return string.IsNullOrEmpty(line) || string.IsNullOrEmpty(line.Trim());
		}

		// ---------------------------------------------------------------------
		public virtual long TotalLinesCount()
		{
			return ContentInLines.Length;
		}

		// ---------------------------------------------------------------------
		public virtual long EmptyLinesCount()
		{
			int emptyLines = 0;
			foreach (string line in ContentInLines)
			{
				if (IsEmptyLine(line))
					emptyLines++;
			} // foreach

			return emptyLines;
		}

		// ---------------------------------------------------------------------
		public virtual long FileSize()
		{
			return (new FileInfo(FileName)).Length;
		}

		// ---------------------------------------------------------------------
		public virtual long CommentarsBlockCount()
		{
			contentWithoutComments = CommentarsBlockCount(ref commentsCount);
			return (long)commentsCount;
		}

		// ---------------------------------------------------------------------
		protected virtual void AddLog(LogEventArgs.NoteType kind, string message)
		{
			if (LogMessage != null)
				LogMessage(/*sender*/this, LogEventArgs.Get(kind, message + " (" + FileName + ")"));
		}

		// ---------------------------------------------------------------------
		// --- ProjectFile Abstract --------------------------------------------
		// ---------------------------------------------------------------------
		public abstract string CommentarsBlockCount(ref long? commentsCount);
	} // public abstract class ProjectFile

	// *************************************************************************
	// *** ProjectFileHelpers **************************************************
	// *************************************************************************
	public static class ProjectFileHelpers
	{
		// ---------------------------------------------------------------------
		// --- ProjectFile Extender --------------------------------------------
		// ---------------------------------------------------------------------
		public static bool ContainsFile(this List<ProjectFile> list, string fileName)
		{
			foreach (ProjectFile projectFile in list)
			{
				if (projectFile.FileName.Equals(fileName, StringComparison.CurrentCultureIgnoreCase))
					return true;
			} // foreach

			return false;
		}
	}

	// *************************************************************************
	// *** HtmlFileType ********************************************************
	// *************************************************************************
	public class HtmlFileType : ProjectFile
	{
		// ---------------------------------------------------------------------
		private const string StartComment = "<!--";
		private const string EndComment = "-->";

		// ---------------------------------------------------------------------
		// --- HtmlFileType Methods --------------------------------------------
		// ---------------------------------------------------------------------
		public override string CommentarsBlockCount(ref long? count)
		{
			if (count != null)
				return Content;

			// TODO Implement JavaScript / CSS comments
			count = 0;
			string contentFile = Content;
			int startIndex = StringUtils.InvariantCultureIndexOf(contentFile, StartComment);
			int endIndex = StringUtils.InvariantCultureIndexOf(contentFile, EndComment);
			while (StringUtils.InvariantCultureIndexOf(contentFile, StartComment) >= 0)
			{
				if (endIndex < startIndex)
				{
					contentFile = contentFile.Remove(endIndex, EndComment.Length);
					AddLog(LogEventArgs.NoteType.Warning, "End comment tag '-->' is placed before start comment tag!");
				} // if
				else if (endIndex < 0)
					contentFile = contentFile.Remove(startIndex);
				else
					contentFile = contentFile.Remove(startIndex, endIndex + EndComment.Length - startIndex);

				count++;
				startIndex = StringUtils.InvariantCultureIndexOf(contentFile, StartComment);
				endIndex = StringUtils.InvariantCultureIndexOf(contentFile, EndComment);
			} // while

			return contentFile;
		}	
	} // public class HtmlFileType

	// *************************************************************************
	// *** CSSFileTyp **********************************************************
	// *************************************************************************
	public class CSSFileTyp : ProjectFile
	{
		// ---------------------------------------------------------------------
		// --- CSSFileTyp Methods ----------------------------------------------
		// ---------------------------------------------------------------------
		public override string CommentarsBlockCount(ref long? count)
		{
			throw new NotSupportedException();
		}
	} // public class CSSFileTyp

	// *************************************************************************
	// *** JavaScriptFileType **************************************************
	// *************************************************************************
	public class JavaScriptFileType : ProjectFile
	{
		// ---------------------------------------------------------------------
		// --- JavaScriptFileType Methods --------------------------------------
		// ---------------------------------------------------------------------
		public override string CommentarsBlockCount(ref long? count)
		{
			throw new NotSupportedException();
		}
	} // public class JavaScriptFileType

	// *************************************************************************
	// *** XmlFileType *********************************************************
	// *************************************************************************
	public class XmlFileType : ProjectFile
	{
		// ---------------------------------------------------------------------
		// --- XmlFileType Methods ---------------------------------------------
		// ---------------------------------------------------------------------
		public override string CommentarsBlockCount(ref long? count)
		{
			throw new NotSupportedException();
		}
	} // public class XmlFileType

} // namespace SourceStat.Analysis