﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace SourceStat.Analysis
{
	// *************************************************************************
	/*
	                             ProjectType ----------------> ProjectFile
                                      |
            	         -------------------------
            	         |                       | 
           	  VisualStudioSolution    VisualStudioProject
	*/

	// *************************************************************************
	internal delegate long CountAction();

	// *************************************************************************
	// *** ActionAttribute *****************************************************
	// *************************************************************************
	public class ActionAttribute : Attribute
	{
		// ---------------------------------------------------------------------
		public string Description = string.Empty;
		public FormatTypes FormatType = FormatTypes.Long;

		// ---------------------------------------------------------------------
		// --- ActionAttribute Constructors ------------------------------------
		// ---------------------------------------------------------------------
		public ActionAttribute(string description)
		{
			Description = description;
		}

		// ---------------------------------------------------------------------
		public ActionAttribute(string description, FormatTypes formatType)
		{
			Description = description;
			FormatType = formatType;
		}

		// ---------------------------------------------------------------------
		// --- ActionAttribute Constructors ------------------------------------
		// ---------------------------------------------------------------------
		public enum FormatTypes
		{
			Long,
			FileSize
		}
	} // public class ActionAttribute

	// *************************************************************************
	// *** ProtocolResult ******************************************************
	// *************************************************************************
	public class ProtocolResult
	{
		// ---------------------------------------------------------------------
		[ActionAttribute("Supported files")]
		public long? SupportedFilesCount = 0;
		[ActionAttribute("Total Lines")]
		public long? TotalLinesCount = 0;
		[ActionAttribute("Blank Lines")]
		public long? EmptyLinesCount = 0;
		[ActionAttribute("Blocks of commentars")]
		public long? CommentarsBlockCount = null;
		[ActionAttribute("Files size", ActionAttribute.FormatTypes.FileSize)]
		public long? FileSizes = 0;
	} // public struct ProtocolResult

	// *************************************************************************
	// *** LogEventArgs ********************************************************
	// *************************************************************************
	public class LogEventArgs : EventArgs
	{
		// ---------------------------------------------------------------------
		public NoteType Kind;
		public string Message;

		// ---------------------------------------------------------------------
		// --- LogEventArgs Enums ----------------------------------------------
		// ---------------------------------------------------------------------
		public enum NoteType
		{
			Hint,
			Warning,
			Error
		} // NoteType

		// ---------------------------------------------------------------------
		// --- LogEventArgs Methods --------------------------------------------
		// ---------------------------------------------------------------------
		public static LogEventArgs Get(NoteType kind, string message)
		{
			return new LogEventArgs()
			{
				Kind = kind,
				Message = message
			};
		}
	} // public class LogEventArgs

	// *************************************************************************
	// *** ProgressEventArgs ***************************************************
	// *************************************************************************
	public class ProgressEventArgs : EventArgs
	{
		// ---------------------------------------------------------------------
		public int Position;
		public int Max;

		// ---------------------------------------------------------------------
		// --- ProgressEventArgs Methods ---------------------------------------
		// ---------------------------------------------------------------------
		public static ProgressEventArgs Get(int position, int max)
		{
			return new ProgressEventArgs()
			{
				Position = position,
				Max = max
			};
		}
	} // public class ProgressEventArgs

	// *************************************************************************
	// *** ProjectType *********************************************************
	// *************************************************************************
	public abstract class ProjectType
	{
		// ---------------------------------------------------------------------
		public string[] ImageExt = new string[] { ".png", ".gif", ".jpeg", ".jpg", ".bmp", ".ico" };
		// ---------------------------------------------------------------------
		private List<ProjectFile> files = null;

		// ---------------------------------------------------------------------
		// --- ProjectType Event Handlers --------------------------------------
		// ---------------------------------------------------------------------
		public event EventHandler<LogEventArgs> LogMessage;

		// ---------------------------------------------------------------------
		public event EventHandler<ProgressEventArgs> IncProgress;

		// ---------------------------------------------------------------------
		// --- ProjectType Properties ------------------------------------------
		// ---------------------------------------------------------------------
		protected List<ProjectFile> Files
		{
			get
			{
				if (files == null)
					files = new List<ProjectFile>();
				
				return files;
			} // get
		}

		// ---------------------------------------------------------------------
		// --- ProjectType Events ----------------------------------------------
		// ---------------------------------------------------------------------
		protected void ProjectFile_LogMessage(object sender, LogEventArgs args)
		{
			AddLog(args.Kind, args.Message);
		}

		// ---------------------------------------------------------------------
		// --- ProjectType Methods ---------------------------------------------
		// ---------------------------------------------------------------------
		public virtual void Run(string projectFileName, ProtocolResult result)
		{
			if (result == null)
				throw new Exception("Internal Error: Result class cannot be empty!");

			if (!File.Exists(projectFileName))
				throw new Exception("File not found!");

			// Get project files
			files = null;	// Reload files
			FillProjectFiles(projectFileName, Files);

			// Analysis
			if (result.SupportedFilesCount != null)
				result.SupportedFilesCount = Files.Count;
			int index = 0;
			foreach (ProjectFile currentFile in Files)
			{
				// Execute actions
				ExecuteAction(ref result.TotalLinesCount, currentFile, new CountAction(currentFile.TotalLinesCount));
				ExecuteAction(ref result.EmptyLinesCount, currentFile, new CountAction(currentFile.EmptyLinesCount));
				ExecuteAction(ref result.CommentarsBlockCount, currentFile, new CountAction(currentFile.CommentarsBlockCount));
				ExecuteAction(ref result.FileSizes, currentFile, new CountAction(currentFile.FileSize));
				
				// Free file contents
				currentFile.InvalidateFileContents();

				// Progress
				OnIncProgress(index, Files.Count - 1);
				index++;
			} // foreach
		}

		// ---------------------------------------------------------------------
		protected virtual void AddLog(LogEventArgs.NoteType kind, string message)
		{
			if (LogMessage != null)
				LogMessage(/*sender*/this, LogEventArgs.Get(kind, message));
		}

		// ---------------------------------------------------------------------
		protected virtual void OnIncProgress(int position, int max)
		{
			if (IncProgress != null)
				IncProgress(/*sender*/this, ProgressEventArgs.Get(position, max));
		}

		// ---------------------------------------------------------------------
		internal virtual void ExecuteAction(ref long? outputValue, ProjectFile currentFile, CountAction countRoutine)
		{
			if (outputValue != null)
			{
				try // catch
				{
					outputValue += countRoutine();
				} // try
				catch (Exception e)
				{
					AddLog(LogEventArgs.NoteType.Error, string.Format("[{0} - {1}()]: {2} ({3})",
						this.GetType().Name, ((System.Reflection.MemberInfo)(((System.Delegate)(countRoutine)).Method)).Name, e.Message, currentFile.FileName));
				} // catch
			} // if
		}

		// ---------------------------------------------------------------------
		protected void AddFile(List<ProjectFile> files, string fileName)
		{
			if (files == null || files.ContainsFile(fileName))
				return;

			if (IsIgnoredFile(fileName))
				AddLog(LogEventArgs.NoteType.Hint, "Skipping (ignored): " + fileName);
			else if (!File.Exists(fileName))
				AddLog(LogEventArgs.NoteType.Warning, "File doesn't exist: " + fileName);
			else
			{
				// Check if is supported type
				ProjectFile projectFile = GetSupportedProjectFileByExt(fileName);
				if (projectFile == null)
					AddLog(LogEventArgs.NoteType.Hint, "Skipping (not supported yet): " + fileName);
				else
					files.Add(projectFile);
			} // else
		}

		// ---------------------------------------------------------------------
		public virtual bool IsSupported(string fileName)
		{
			string fileExt = Path.GetExtension(fileName).ToLower();
			return GetExtensions().Contains(fileExt);
		}

		// ---------------------------------------------------------------------
		// --- ProjectType Abstract Methods ------------------------------------
		// ---------------------------------------------------------------------
		public abstract void FillProjectFiles(string projectContent, List<ProjectFile> files);

		// ---------------------------------------------------------------------
		public abstract string[] GetExtensions();

		// ---------------------------------------------------------------------
		protected abstract ProjectFile GetSupportedProjectFileByExt(string projectFileName);

		// ---------------------------------------------------------------------
		protected abstract bool IsIgnoredFile(string fileName);

		// ---------------------------------------------------------------------
		public abstract string GetDescription();
	} // public abstract class ProjectType

} // namespace SourceStat.Analysis