﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Xml.Linq;
using SourceStat.Utils;

namespace SourceStat.Analysis.VS2005_2010
{
	// *************************************************************************
	// *** VS2008Project *******************************************************
	// *************************************************************************
	/// <summary>
	/// Visual Studio 2005 / 2008
	/// </summary>
	public class VS2005_2010Project : ProjectType
	{
		// ---------------------------------------------------------------------
		private const string SchemaNameSpace = "http://schemas.microsoft.com/developer/msbuild/2003";

		// ---------------------------------------------------------------------
		// --- VS2005_2008Project Methods --------------------------------------
		// ---------------------------------------------------------------------
		public override void FillProjectFiles(string projectFileName, List<ProjectFile> files)
		{
			XDocument document;
			try // catch
			{
				document = XDocument.Load(projectFileName);
			} // try
			catch (Exception e)
			{
				throw new Exception("Invalid VS2008 project file format: " + projectFileName + " [" + e.Message + "]");
			} // catch

			// Select project files
			string solutionPath = Path.GetDirectoryName(projectFileName);
			foreach (XElement element in document.Descendants(XName.Get("Compile", SchemaNameSpace)))
				AddFilesFromNode(files, element, solutionPath, projectFileName);
			foreach (XElement element in document.Descendants(XName.Get("Content", SchemaNameSpace)))
				AddFilesFromNode(files, element, solutionPath, projectFileName);
			foreach (XElement element in document.Descendants(XName.Get("EmbeddedResource", SchemaNameSpace)))
				AddFilesFromNode(files, element, solutionPath, projectFileName);
		}

		// ---------------------------------------------------------------------
		/// <summary>
		/// Return instance of ProjectFile by file extenstion.
		/// </summary>
		/// <param name="projectFileName"></param>
		/// <returns></returns>
		protected override ProjectFile GetSupportedProjectFileByExt(string projectFileName)
		{
			// TODO	DBML / XAML support
			switch (Path.GetExtension(projectFileName).ToLower())
			{
				case ".cs":
				{
					VSCSharpFileType fileType = new VSCSharpFileType() { FileName = projectFileName };
					fileType.LogMessage += ProjectFile_LogMessage;
					return fileType;
				} // cs

				case ".vb":
				{
					VSVisualBasicFileType fileType = new VSVisualBasicFileType() { FileName = projectFileName };
					fileType.LogMessage += ProjectFile_LogMessage;
					return fileType;
				} // cs

				case ".htm":
				case ".html":
				case ".aspx":
				case ".ascx":
				case ".skin":
				case ".master":
				{
					HtmlFileType fileType = new HtmlFileType() { FileName = projectFileName };
					fileType.LogMessage += ProjectFile_LogMessage;
					return fileType;
				} // html

				case ".css":
				{
					CSSFileTyp fileType = new CSSFileTyp() { FileName = projectFileName };
					fileType.LogMessage += ProjectFile_LogMessage;
					return fileType;
				} // html

				case ".js":
				{
					JavaScriptFileType fileType = new JavaScriptFileType() { FileName = projectFileName };
					fileType.LogMessage += ProjectFile_LogMessage;
					return fileType;
				} // html

				case ".xaml":
				{
					XmlFileType fileType = new XmlFileType() { FileName = projectFileName };
					fileType.LogMessage += ProjectFile_LogMessage;
					return fileType;
				} // html
			} // switch

			return null;
		}

		// ---------------------------------------------------------------------
		protected override bool IsIgnoredFile(string fileName)
		{
			if (string.IsNullOrEmpty(fileName))
				return false;

			fileName = fileName.ToLower();
			string ext = Path.GetExtension(fileName);
			return
				ImageExt.Contains(ext) ||
				".dbml".Equals(ext) ||
				".asax".Equals(ext) ||
				".ashx".Equals(ext) ||
				".resx".Equals(ext) ||
				".xap".Equals(ext) ||
				".settings".Equals(ext) ||
				".config".Equals(ext) ||
				".asmx".Equals(ext) ||
				".svc".Equals(ext) ||
				".map".Equals(ext) ||
				".svcmap".Equals(ext) ||
				// Possible resources
				".ttf".Equals(ext) ||
				".txt".Equals(ext) ||
				".xml".Equals(ext) ||
				".xsd".Equals(ext) ||
				".rdlc".Equals(ext) ||
				".myapp".Equals(ext) ||
				// Ignore WSDL / designer files
				fileName.EndsWith("reference.cs") ||
				fileName.EndsWith(".designer.cs");
		}

		// ---------------------------------------------------------------------
		public override string[] GetExtensions()
		{
			return new string[] { ".csproj", ".vbproj" };
		}

		// ---------------------------------------------------------------------
		protected virtual void AddFilesFromNode(List<ProjectFile> files, XElement element, string solutionPath, string projectFileName)
		{
			if (element == null)
				return;

			XAttribute fileNameAttribute = element.Attribute(XName.Get("Include"));
			if (fileNameAttribute != null)
			{
				// Main file
				element.Attribute(XName.Get("Include"));				
				if (fileNameAttribute != null)
					AddFile(files, Path.Combine(solutionPath, fileNameAttribute.Value));

				// Attached files
				foreach (XElement innerElement in element.Descendants(XName.Get("DependentUpon", SchemaNameSpace)))
				{
					// Use relative path from parent element too
					string fullPath = Path.Combine(solutionPath, Path.GetDirectoryName(fileNameAttribute.Value));
					AddFile(files, Path.Combine(fullPath, innerElement.Value));
				} // foreach
			} // if
			else
				AddLog(LogEventArgs.NoteType.Warning, "Problem with parsing file name from project file: " + projectFileName);
		}

		// ---------------------------------------------------------------------
		public override string GetDescription()
		{
			return "VS Project (2005/08)";
		}

		// ---------------------------------------------------------------------
		public override bool IsSupported(string fileName)
		{
			return true;	// ???
		}
	} // public class VS2005_2008Project

	// *************************************************************************
	// *** VS2005_2008Solution *************************************************
	// *************************************************************************
	public class VS2005_2010Solution : ProjectType
	{
		// ---------------------------------------------------------------------
		private VS2005_2010Project project = null;

		// ---------------------------------------------------------------------
		// --- VS2005_2008Solution Events --------------------------------------
		// ---------------------------------------------------------------------
		protected VS2005_2010Project Project
		{
			get
			{
				if (project == null)
				{
					project = new VS2005_2010Project();
					project.LogMessage += Project_LogMessage;
				} // if

				return project;
			} // get
		}

		// ---------------------------------------------------------------------
		// --- VS2008Solution Eevents ------------------------------------------
		// ---------------------------------------------------------------------
		public void Project_LogMessage(object sender, LogEventArgs args)
		{
			AddLog(args.Kind, args.Message);
		}

		// ---------------------------------------------------------------------
		// --- VS2005_2008Solution Methods -------------------------------------
		// ---------------------------------------------------------------------
		public override void FillProjectFiles(string solutionFileName, List<ProjectFile> files)
		{
			List<string> projects = new List<string>();
			if (Project.GetExtensions().Contains(Path.GetExtension(solutionFileName).ToLower()))
				projects.Add(solutionFileName);
			else
			{
				// Extract lines contaned project names
				List<string> projectDefinitions = new List<string>(
					File.ReadAllText(solutionFileName).Split(new string[] { "Project(" }, StringSplitOptions.RemoveEmptyEntries));
				projectDefinitions.RemoveAt(0);

				// Extract project names
				char[] paramsDelimiters = new char[] { ',' };
				string solutionPath = Path.GetDirectoryName(solutionFileName);
				foreach (string projectDefinition in projectDefinitions)
				{
					string[] projectParams = projectDefinition.Split(paramsDelimiters, StringSplitOptions.RemoveEmptyEntries);
					if (projectParams.Count() < 1)
						AddLog(LogEventArgs.NoteType.Error, "Invalid solution format - cannot detect project file!");
					else
					{
						// Remove quatas
						string relativeProjectPath = projectParams[1].Replace("\"", string.Empty).Trim();
						projects.Add(Path.Combine(solutionPath, relativeProjectPath));
					} // else
				} // foreach
			} // else

			// Get files
			foreach (string projectFileName in projects)
			{
				if (!File.Exists(projectFileName))
					AddLog(LogEventArgs.NoteType.Warning, "Project file '" + projectFileName + "' from solution doesn't exist!");
				else if (!Project.GetExtensions().Contains(Path.GetExtension(projectFileName).ToLower()))
					AddLog(LogEventArgs.NoteType.Hint, "Skipping (not supported yet): '" + projectFileName + "'.");
				else
				{
					// Parse by project instance
					Project.FillProjectFiles(projectFileName, files);
				} // else
			} // foreach
		}

		// ---------------------------------------------------------------------
		public override string[] GetExtensions()
		{
			// VS Project supported ext.
			List<string> extensions = new List<string>(Project.GetExtensions());

			// VS Solution supported ext.
			extensions.Add(".sln");

			return extensions.ToArray();
		}

		// ---------------------------------------------------------------------
		protected override ProjectFile GetSupportedProjectFileByExt(string projectFileName)
		{
			// SLN is using PROJ for wokring with files
			throw new NotImplementedException();
		}

		// ---------------------------------------------------------------------
		protected override bool IsIgnoredFile(string fileName)
		{
			// SLN is using PROJ for wokring with files
			throw new NotImplementedException();
		}

		// ---------------------------------------------------------------------
		public override string GetDescription()
		{
			return "VS Solution (2005/08)";
		}

		// ---------------------------------------------------------------------
		public override bool IsSupported(string fileName)
		{
			if (base.IsSupported(fileName))
			{
				if (Project.IsSupported(fileName))
					return true;

				string solution = File.ReadAllText(fileName);
				int indexVersion = StringUtils.InvariantCultureIndexOf(solution, "Format Version");
				if (indexVersion >= 0)
				{
					solution = solution.Substring(indexVersion);
					solution = solution.Remove(StringUtils.InvariantCultureIndexOf(solution, "\r\n"));
					return (solution.EndsWith("9.00") || solution.EndsWith("10.00"));
				} // if
			} // if

			return false;
		}
	} // public class VS2005_2008Solution

	// *************************************************************************
	// *** VSCSharpFileType ****************************************************
	// *************************************************************************
	public class VSCSharpFileType : ProjectFile
	{
		// ---------------------------------------------------------------------
		private const string StartBlockComment = "/*";
		private const string EndBlockComment = "*/";
		private const string LineCommentPreffix = "//";

		// ---------------------------------------------------------------------
		// --- VSCSharpFileType Methods ----------------------------------------
		// ---------------------------------------------------------------------
		public override string CommentarsBlockCount(ref long? count)
		{
			if (count != null)
				return Content;

			count = 0;
			string contentFile = Content;
			int startIndexBlock = StringUtils.InvariantCultureIndexOf(contentFile, StartBlockComment);
			int startIndexLine = StringUtils.InvariantCultureIndexOf(contentFile, LineCommentPreffix);
			while (startIndexBlock >= 0 || startIndexLine >= 0)
			{
				bool paralelExistence = (startIndexBlock >= 0 && startIndexLine >= 0);
				if ((paralelExistence && startIndexBlock > startIndexLine) || startIndexBlock < 0)
				{
					RemoveFirstLineComment(ref contentFile, startIndexLine);
					count++;
				} // if
				else if ((paralelExistence && startIndexBlock < startIndexLine) || startIndexLine < 0)
				{
					RemoveFirstBlockComment(ref contentFile, startIndexBlock);
					count++;
				} // else if
				else
					Debug.Assert(false, "Unexpected state!");

				// Continue in serach
				startIndexBlock = StringUtils.InvariantCultureIndexOf(contentFile, StartBlockComment);
				startIndexLine = StringUtils.InvariantCultureIndexOf(contentFile, LineCommentPreffix);
			} // while

			return contentFile;
		}

		// ---------------------------------------------------------------------
		private void RemoveFirstBlockComment(ref string contentFile, int startIndex)
		{
			int endIndex = StringUtils.InvariantCultureIndexOf(contentFile, EndBlockComment/*, startIndex*/);
			if (endIndex < 0)
				contentFile = contentFile.Remove(startIndex);
			else
				contentFile = contentFile.Remove(startIndex, endIndex + EndBlockComment.Length - startIndex);
		}

		// ---------------------------------------------------------------------
		private void RemoveFirstLineComment(ref string contentFile, int startIndex)
		{
			int endIndex = contentFile.IndexOf('\n', startIndex);
			if (endIndex < startIndex)
				contentFile = contentFile.Remove(startIndex);
			else
				contentFile = contentFile.Remove(startIndex, endIndex - startIndex);
		}
	} // public class VSCSharpFileType

	// *************************************************************************
	// *** VSVisualBasicFileType ***********************************************
	// *************************************************************************
	public class VSVisualBasicFileType : ProjectFile
	{
		// ---------------------------------------------------------------------
		// --- VSVisualBasicFileType Methods -----------------------------------
		// ---------------------------------------------------------------------
		public override string CommentarsBlockCount(ref long? count)
		{
			throw new NotSupportedException();
		}
	} // public class VSCSharpFileType

} // namespace SourceStat.Analysis.VS2008