﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using SourceStat.Analysis;
using SourceStat.Utils;

namespace SourceStat
{
	// *************************************************************************
	// *** FormMain ************************************************************
	// *************************************************************************
	public partial class FormMain : Form
	{
		// ---------------------------------------------------------------------
		private static readonly ProjectType[] SupportedProjectTypes = new ProjectType[]
		{
			new SourceStat.Analysis.VS2005_2010.VS2005_2010Solution()
		};
		// ---------------------------------------------------------------------
		private string dialogFilter;

		// ---------------------------------------------------------------------
		// --- FormMain Constructor --------------------------------------------
		// ---------------------------------------------------------------------
		public FormMain()
		{
			InitializeComponent();

#if DEBUG
			// textBoxFileName.Text = @"F:\EES\EES_CS.net\Source\EES.sln";
			// textBoxFileName.Text = @"F:\EES\EES_CS.net\Source\Web\EES_Web\Commercial\Commercial.csproj";
			// textBoxFileName.Text = @"F:\EES\EES_CS.net\Source\Web\EES_Web\Panels\Panels.csproj";
			// textBoxFileName.Text = @"F:\EES\EES_CS.net\Source\EES_report\EES_report.csproj";
			// textBoxFileName.Text = @"F:\!\VS2003\SearcherAddIn\SearcherAddIn.sln";
#endif // DEBUG

			// Init supported projects
			List<string> filters = new List<string>();
			foreach (ProjectType project in SupportedProjectTypes)
			{
				project.LogMessage += Solution_LogMessage;
				project.IncProgress += Solution_IncProgress;
				
				// Create filter with supported types
				List<string> extenstions = new List<string>(project.GetExtensions());
				for (int extIndex = 0; extIndex < extenstions.Count; extIndex++)
					extenstions[extIndex] = "*" + extenstions[extIndex];
				// Add filter
				filters.Add(string.Format("{0} files ({1})|{2}",
					project.GetDescription(), string.Join(", ", extenstions.ToArray()), string.Join(";", extenstions.ToArray())));
			} // foreach
			dialogFilter = string.Join("|", filters.ToArray());

			// Fill the possible actions
			ProtocolResultHelper.InitActionsList(checkedListBoxActions);

			// Display version in caption of window
			Text += " [" + System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString() + "]";
		}

		// ---------------------------------------------------------------------
		// --- FormMain Events -------------------------------------------------
		// ---------------------------------------------------------------------
		private void ButtonOpenDialog_Click(object sender, EventArgs e)
		{
			using (OpenFileDialog dialog = new OpenFileDialog())
			{
				dialog.Multiselect = false;
				dialog.Filter = dialogFilter;

				if (dialog.ShowDialog() == DialogResult.OK)
					textBoxFileName.Text = dialog.FileName;
			} // using
		}

		// ---------------------------------------------------------------------
		private void ButtonExecute_Click(object sender, EventArgs e)
		{
			Cursor = Cursors.WaitCursor;
			try // finally
			{
				listBoxResults.Items.Clear();
				listBoxLog.Items.Clear();

				// Prepare analysis tool class
				string fileName = textBoxFileName.Text;
				ProjectType project = null;
				foreach (ProjectType supportedProject in SupportedProjectTypes)
				{
					if (supportedProject.IsSupported(fileName))
					{
						project = supportedProject;
						break;
					} // if
				} // foreach			
				if (project == null)
					throw new NotSupportedException("Project is not supported!");

				// Do analysis
				int startTick = Environment.TickCount;
				ProtocolResult solutionResults = ProtocolResultHelper.GetProtocolResult(checkedListBoxActions);
				project.Run(fileName, solutionResults);
				int endTick = Environment.TickCount;

				// Get time
				long diffTick = endTick - startTick;
				long hours = (diffTick / (3600 * 999)) % 24;
				long minutes = (diffTick / (60 * 999)) % 60;
				long seconds = (diffTick / 999) % 60;

				// Display results
				listBoxResults.Items.Add(string.Format("Solution (Project) [{0}:{1}:{2}]: {3}",
					hours.ToString("00"), minutes.ToString("00"), seconds.ToString("00"), fileName));
				listBoxResults.Items.Add("Type: " + solutionResults);
				ProtocolResultHelper.DisplayResult(listBoxResults.Items, solutionResults);
				listBoxResults.Items.Add(string.Empty);
				// Extended stat
				listBoxResults.Items.Add("Stat: ");
				if (solutionResults.TotalLinesCount != null && solutionResults.EmptyLinesCount != null)
				{
					string value = (solutionResults.EmptyLinesCount > 0 ?
						string.Format("{0:#,0}", solutionResults.TotalLinesCount / solutionResults.EmptyLinesCount) : "0");
					listBoxResults.Items.Add("Lines / Empty lines: " + value);
				} // if
				if (solutionResults.TotalLinesCount != null && solutionResults.SupportedFilesCount != null)
				{
					string value = (solutionResults.SupportedFilesCount > 0 ?
						string.Format("{0:#,0}", solutionResults.TotalLinesCount / solutionResults.SupportedFilesCount) : "0");
					listBoxResults.Items.Add("Avrg lines in file: " + value);
				} // if
				if (solutionResults.FileSizes != null && solutionResults.SupportedFilesCount != null)
				{
					string value = (solutionResults.SupportedFilesCount > 0 ?
						string.Format(FileSizeProvider.MyFileSizeProvider, "{0:AUTO}", solutionResults.FileSizes / solutionResults.SupportedFilesCount) : "0");
					listBoxResults.Items.Add("Avrg file size: " + value);
				} // if

				// Finalize process
				progressBar.Value = 0;
			} // finally
			finally
			{
				Cursor = Cursors.Default;
			} // finally
		}

		// ---------------------------------------------------------------------
		public void Solution_LogMessage(object sender, LogEventArgs args)
		{
			if (args.Kind == LogEventArgs.NoteType.Hint && !checkBoxLogHints.Checked)
				return;

			listBoxLog.Items.Add(args.Kind.ToString() + "\t" + args.Message);
			listBoxLog.SelectedIndex = (listBoxLog.Items.Count - 1);
			Application.DoEvents();
		}

		// ---------------------------------------------------------------------
		public void Solution_IncProgress(object sender, ProgressEventArgs args)
		{
			progressBar.Maximum = args.Max;
			progressBar.Value = args.Position;
			Application.DoEvents();
		}
	} // public partial class FormMain

} // namespace SourceStat